@@ -120,7 +120,7 @@ def get_target_program(project_path, aos_sdk_path, target):
     # copyp os default dir
     for d in default_dir_list:
         src = os.path.join(aos_sdk_path, d)
-        dst = os.path.join(project_path, 'os', d)
+        dst = os.path.join(project_path, 'aos', d)
         copy_directory(src, dst)
 
     # get all component through config.mk
@@ -139,7 +139,7 @@ def get_target_program(project_path, aos_sdk_path, target):
         sm = sm.strip()
         if sm:
             copy_dir_dict[os.path.abspath(os.path.dirname(os.path.join(aos_sdk_path, sm)))] = os.path.abspath(
-                os.path.dirname(os.path.join(project_path, 'os', sm)))
+                os.path.dirname(os.path.join(project_path, 'aos', sm)))
 
     # sdk prebuild libraries dir
     sdk_prebuild_libs = config_dict['AOS_SDK_PREBUILT_LIBRARIES'].split(' ')
@@ -147,7 +147,7 @@ def get_target_program(project_path, aos_sdk_path, target):
         sp = sp.strip()
         if sp:
             copy_dir_dict[os.path.abspath(os.path.dirname(os.path.join(aos_sdk_path, sp)))] = os.path.abspath(
-                os.path.dirname(os.path.join(project_path, 'os', sp)))
+                os.path.dirname(os.path.join(project_path, 'aos', sp)))
 
     # sdk include dir
     sdk_includes = config_dict['AOS_SDK_INCLUDES'].split('-I')
@@ -155,14 +155,14 @@ def get_target_program(project_path, aos_sdk_path, target):
         si = si.strip()
         if si:
             copy_dir_dict[os.path.abspath(os.path.join(aos_sdk_path, si))] = os.path.abspath(
-                os.path.join(project_path, 'os', si))
+                os.path.join(project_path, 'aos', si))
 
     # all component dir
     components = config_dict[COMPONENTS].split(' ')
     for co in components:
         # component dir
         src_dir = os.path.abspath(os.path.join(aos_sdk_path, config_dict[co + '_LOCATION']))
-        dst_dir = os.path.abspath(os.path.join(project_path, 'os', config_dict[co + '_LOCATION']))
+        dst_dir = os.path.abspath(os.path.join(project_path, 'aos', config_dict[co + '_LOCATION']))
         copy_dir_dict[src_dir] = dst_dir
 
         # component c file dir
@@ -179,7 +179,7 @@ def get_target_program(project_path, aos_sdk_path, target):
             si = si.strip()
             if si:
                 copy_dir_dict[os.path.abspath(os.path.join(aos_sdk_path, si))] = os.path.abspath(
-                    os.path.join(project_path, 'os', si))
+                    os.path.join(project_path, 'aos', si))
 
     # application dir, must last copy because app target dir isn't the same as src dir
     app = config_dict['APP']
@@ -202,7 +202,7 @@ def get_target_program(project_path, aos_sdk_path, target):
     with open(os.path.join(aos_sdk_path, 'out', target, 'dependency.json'), 'r') as fin:
         dep_dict = json.loads(fin.read())
         for co in components:
-            location = os.path.join('os', relpath(aos_sdk_path, os.path.abspath(
+            location = os.path.join('aos', relpath(aos_sdk_path, os.path.abspath(
                 os.path.join(aos_sdk_path, config_dict[co + '_LOCATION'])))) if co != app else app
             if os.path.join('out', target) not in location:
                 ref = ''
@@ -215,7 +215,7 @@ def get_target_program(project_path, aos_sdk_path, target):
     with open(os.path.join(project_path, COMPONENTS), 'w') as fout:
         fout.write(json.dumps(component_dep, sort_keys=True, indent=4))
 
-    pd.set_cfg(OS_PATH, os.path.join(project_path, 'os'))
+    pd.set_cfg(OS_PATH, os.path.join(project_path, 'aos'))
     pd.set_cfg(PATH_TYPE, 'program')
 
 
@@ -317,7 +317,7 @@ def new(name, path='.', scm='git', program=False, component=False, aoslib=False,
             if repo.scm.name != scm:
                 error(
                     "A repository already exists in \"%s\" based on %s. Please select a different name or location." % (
-                    d_path, scm), 1)
+                        d_path, scm), 1)
         else:
             repo_scm = [s for s in scms.values() if s.name == scm.lower()]
             if not repo_scm:
@@ -380,7 +380,7 @@ def new(name, path='.', scm='git', program=False, component=False, aoslib=False,
         with cd(d_path):
             with open(os.path.join(d_path, name + '.mk'), 'w') as fm:
                 makefile_content = 'NAME := %s\n$(NAME)_SOURCES := %s\n$(NAME)_COMPONENTS :=\n$(NAME)_INCLUDES := .\n' % (
-                name, name + '.c')
+                    name, name + '.c')
                 fm.write(makefile_content)
 
             with open(os.path.join(d_path, name + '.c'), 'w') as fc:
@@ -507,7 +507,7 @@ def import_(url, path=None, ignore=False, depth=None, protocol=None, top=True):
 
     # translate 'aos' to https://code.aliyun.com/aos
     if not Repo.isurl(url) and not os.path.exists(url):
-        url = aos_base_url + '/' + url + '.git'
+        url = AOS_COMPONENT_BASE_URL + '/' + url + '.git'
 
     repo = Repo.fromurl(url, path)
     if top:
@@ -516,18 +516,18 @@ def import_(url, path=None, ignore=False, depth=None, protocol=None, top=True):
             error(
                 "Cannot import program in the specified location \"%s\" because it's already part of a program \"%s\".\n"
                 "Please change your working directory to a different location or use \"aos add\" to import the URL as a component." % (
-                os.path.abspath(repo.path), p.name), 1)
+                    os.path.abspath(repo.path), p.name), 1)
 
     protocol = Program().get_cfg('PROTOCOL', protocol)
 
-    if os.path.isdir(repo.path) and len(os.listdir(repo.path)) > 1:
+    if os.path.isdir(repo.path) and len(os.listdir(repo.path)) > 1 and is_path_component(repo.path):
         # error("Directory \"%s\" is not empty. Please ensure that the destination folder is empty." % repo.path, 1)
-        info("Directory \"%s\" is not empty. Please ensure that the destination folder is empty." % repo.path, 1)
-        return
+        info("Directory \"%s\" is a component." % repo.path, 1)
+        return True
 
     text = "Importing program" if top else "Add component"
     action("%s \"%s\" from \"%s\"%s" % (
-    text, os.path.basename(repo.path), formaturl(repo.url, protocol), ' at ' + (repo.revtype(repo.rev, True))))
+        text, os.path.basename(repo.path), formaturl(repo.url, protocol), ' at ' + (repo.revtype(repo.rev, True))))
     if repo.clone(repo.url, repo.path, rev=repo.rev, depth=depth, protocol=protocol):
         with cd(repo.path):
             Program(repo.path).set_root()
@@ -540,16 +540,12 @@ def import_(url, path=None, ignore=False, depth=None, protocol=None, top=True):
                     err = err + (
                             "\nThe --depth option might prevent fetching the whole revision tree and checking out %s." % (
                     repo.revtype(repo.rev, True)))
-                if ignore:
-                    warning(err)
-                else:
-                    error(err, e[0])
+                warning(err)
+                return False
     else:
         err = "Unable to clone repository (%s)" % url
-        if ignore:
-            warning(err)
-        else:
-            error(err, 1)
+        warning(err)
+        return False
 
     if os.path.isdir(repo.path):
         repo.sync()
@@ -575,9 +571,10 @@ def import_(url, path=None, ignore=False, depth=None, protocol=None, top=True):
         with cd(repo.path):
             deploy(ignore=ignore, depth=depth, protocol=protocol, top=False)
 
-
 #    if top:
 #        Program(repo.path).post_action()
+    return True
+
 
 def get_component_official_info(official_component_dict, name):
     component_makefile = ''
@@ -609,7 +606,7 @@ def get_component_dependencies(makefile):
                 line = line.strip('\n')
                 while line.endswith('\\'):
                     line = line[:-1] + next(fin).rstrip('\n')
-                if '$(NAME)_COMPONENTS' in line and not line.startswith('#'):
+                if '$(NAME)_COMPONENTS' in line and not line.strip().startswith('#'):
                     line = line[line.rfind('=') + 1:].strip()
                     components = re.split(r'\s+', line)
                     for co in components:
@@ -618,12 +615,29 @@ def get_component_dependencies(makefile):
     return dependencies
 
 
-def add_components(name):
+def get_component_includes(makefile):
+    includes = []
+    if makefile and makefile.endswith('.mk') and os.path.isfile(makefile):
+        with open(makefile) as fin:
+            for line in fin:
+                line = line.strip('\n')
+                while line.endswith('\\'):
+                    line = line[:-1] + next(fin).rstrip('\n')
+                if '$(NAME)_INCLUDES' in line and not line.strip().startswith('#'):
+                    line = line[line.rfind('=') + 1:].strip()
+                    heads = re.split(r'\s+', line)
+                    for head in heads:
+                        if head:
+                            includes.append(head)
+    return includes
+
+
+def add_components_local(name):
     official_component_dict = get_all_official_component()
     component_name, component_makefile = get_component_official_info(official_component_dict, name)
 
     if component_name not in official_component_dict:
-        warning("Component %s must be in AliOS-Things." % (name))
+        warning("Component %s isn\'t in AliOS-Things." % (name))
         # error("component (%s) must be in AliOS-Things." % (name))
         return False
 
@@ -631,13 +645,33 @@ def add_components(name):
         program_component = json.loads(fin.read())
     if component_name in program_component:
         # action("component (%s) is already in current program." % (name))
-        return False
+        return True
 
     sdk_component_dir = os.path.dirname(component_makefile)
     copy_directory(sdk_component_dir, os.path.abspath(
-        os.path.join(os.getcwd(), 'os', relpath(Global().get_cfg(AOS_SDK_PATH), sdk_component_dir))), True)
+        os.path.join(os.getcwd(), 'aos', relpath(Global().get_cfg(AOS_SDK_PATH), sdk_component_dir))), True)
 
-    location = os.path.join('os', relpath(Global().get_cfg(AOS_SDK_PATH), sdk_component_dir))
+    includes = get_component_includes(component_makefile)
+
+    copy_dir_dict = {}
+    for include in includes:
+        src_dir = os.path.join(os.path.dirname(component_makefile), include)
+        dst_dir = os.path.join(os.getcwd(), 'aos', relpath(Global().get_cfg(AOS_SDK_PATH), src_dir))
+        copy_dir_dict[src_dir] = dst_dir
+
+    aos_sdk_path = Global().get_cfg(AOS_SDK_PATH)
+    order_copy_dir_dict = collections.OrderedDict(sorted(copy_dir_dict.items()))
+    for src_dir, dst_dir in order_copy_dir_dict.items():
+        is_component = is_path_component(src_dir)
+        while not is_component and src_dir != aos_sdk_path:
+            src_dir = os.path.dirname(src_dir)
+            dst_dir = os.path.dirname(dst_dir)
+            is_component = is_path_component(src_dir)
+
+        if is_component:
+            copy_directory(src_dir, dst_dir)
+
+    location = os.path.join('aos', relpath(Global().get_cfg(AOS_SDK_PATH), sdk_component_dir))
     dependencies = get_component_dependencies(component_makefile)
     origin = 'local'
 
@@ -653,7 +687,46 @@ def add_components(name):
         fout.write(json.dumps(program_component, sort_keys=True, indent=4))
 
     for dep in dependencies:
-        add_components(dep)
+        add_components_local(dep)
+
+    return True
+
+
+def add_components_remote(base_url, scm, name):
+    is_local = add_components_local(name)
+    if not is_local:
+        # clone from the same as name base url
+        lib = Repo.fromurl(base_url + '/' + name + '.' + scm.name)
+        is_import_success = import_(lib.fullurl, lib.path, True, None, None, False)
+        if not is_import_success and base_url != AOS_COMPONENT_BASE_URL:
+            # clone from aos component url
+            lib = Repo.fromurl(AOS_COMPONENT_BASE_URL + '/' + name + '.' + scm.name)
+            is_import_success = import_(lib.fullurl, lib.path, True, None, None, False)
+
+        if not is_import_success:
+            error('Component %s can\'t find in AOS_SDK_PATH && %s && %s' % (name, base_url, AOS_COMPONENT_BASE_URL))
+
+        lib.sync()
+
+        pd = Program(os.getcwd())
+        with open(COMPONENTS, 'r') as fin:
+            program_component = json.loads(fin.read())
+        if lib.name in program_component:
+            action("Component %s has added in current program." % (lib.name))
+            return True
+        else:
+            location = relpath(pd.get_cfg(PROGRAM_PATH), lib.path)
+            dependencies = get_component_dependencies(os.path.join(location, lib.name + '.mk'))
+            ref = get_component_ref(lib)
+            origin = 'remote'
+            action('Add component %s (%s) in %s' % (lib.name, origin, location))
+            program_component[lib.name] = {'location': location, 'dependencies': dependencies, 'status': 'used',
+                                           'origin': origin, 'ref': ref}
+            with open(COMPONENTS, 'w') as fout:
+                fout.write(json.dumps(program_component, sort_keys=True, indent=4))
+
+            for dep in dependencies:
+                add_components_remote(lib.url[:lib.url.rfind('/')], lib.scm, dep)
 
     return True
 
@@ -675,7 +748,7 @@ def add_components(name):
 def add(name, path=None, ignore=False, depth=None, protocol=None, top=True):
     # translate 'aos' to https://code.aliyun.com/aos
     # if not Repo.isurl(url) and not os.path.exists(url):
-    #     url = aos_base_url+'/'+url+'.git'
+    #     url = AOS_COMPONENT_BASE_URL+'/'+url+'.git'
 
     if Repo.isurl(name):
         # repo = Repo.fromrepo()
@@ -707,6 +780,9 @@ def add(name, path=None, ignore=False, depth=None, protocol=None, top=True):
                     with open(COMPONENTS, 'w') as fout:
                         fout.write(json.dumps(program_component, sort_keys=True, indent=4))
 
+                    for dep in dependencies:
+                        add_components_remote(lib.url[:lib.url.rfind('/')], lib.scm, dep)
+
         # lib.write()
         # repo.add(lib.lib)
 
@@ -723,7 +799,7 @@ def add(name, path=None, ignore=False, depth=None, protocol=None, top=True):
                 action("Component %s has added in current program." % (name))
                 return
             else:
-                add_components(name)
+                add_components_local(name)
 
 
 #    if top:
@@ -1145,13 +1221,13 @@ def make():
             root_dir = ['example', 'board', 'kernel', 'platform', 'utility', 'framework', 'tools', 'test', 'device',
                         'security']
             for component, info in json_components.items():
-                location = relpath('os', info['location']) if info['location'].startswith('os') else info['location']
+                location = relpath('aos', info['location']) if info['location'].startswith('aos') else info['location']
                 for root in root_dir:
-                    if location.startswith(root):
+                    if location.startswith(root + os.sep) and len(location) >= (len(root) + 1):
                         location = location[len(root) + 1:]
                 program_components += '@' + location if program_components else location
-        program_components = 'DEFAULT_ALL_COMPONENTS=' + program_components
 
+        program_components = 'DEFAULT_ALL_COMPONENTS=' + program_components
         app_path = 'APPDIR=' + os.path.dirname(pd.get_cfg(APP_PATH))
         os_path = pd.get_cfg(OS_PATH)
         out_path = 'BUILD_DIR=' + os.path.join(os.getcwd(), 'out')
