@@ -1,5 +1,5 @@
 # Application version
-ver = '0.0.13'
+ver = '0.0.14'
 
 # Default paths to Mercurial and Git
 hg_cmd = 'hg'
@@ -33,8 +33,6 @@ regex_aos_url = r'^(https?)://([\w\-\.]*aos\.(co\.uk|org|com))/(users|teams)/([\
 # aos sdk builds url
 regex_build_url = r'^(https?://([\w\-\.]*aos\.(co\.uk|org|com))/(users|teams)/([\w\-]{1,32})/(repos|code)/([\w\-]+))/builds/?([\w\-]{6,40}|tip)?/?$'
 
-# base url for all aos related repos (used as sort of index)
-aos_base_url = 'https://code.aliyun.com/aos'
 # default aos url
 aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
 # default aos component url
@@ -62,3 +60,4 @@ OS_PATH = 'os_path'
 OS_NAME = 'AliOS-Things'
 PATH_TYPE = 'path_type'
 COMPONENTS = 'COMPONENTS'
+AOS_COMPONENT_BASE_URL = 'https://github.com/AliOS-Things'
